// wind_misc_utils.cpp

#include "Arduino.h"
#include "wind_misc_utils.h"

void appendf(String *output, char *fmt, ...) {
  char buf[128]; // resulting string limited to 128 chars
  va_list args;
  va_start (args, fmt );
  vsnprintf(buf, 128, fmt, args);
  va_end (args);
  output->concat(buf);
}

void getTimeString(uint32 msecs, String *output) {
  int time_min = msecs / 1000 / 60;
  int time_secs = msecs / 1000 % 60;
  int time_msecs = msecs % 1000;
  appendf(output, "%02d:%02d.%03d", time_min, time_secs, time_msecs);
}

void joinWiFiNetwork(char* ssid, char* password) {
  // Connect to WiFi network
  WiFi.begin(ssid, password);
  Serial.print("\n\r \n\rConnecting to WiFi.");
  // Wait for connection
  while (WiFi.status() != WL_CONNECTED) {
    delay(100);
    Serial.println(".");
  }
  Serial.println("");

  Serial.print("Connected to ");
  Serial.println(ssid);

  // Print the IP address
  Serial.print("Use this URL to connect: ");
  Serial.print("http://");
  Serial.print(WiFi.localIP());
  Serial.println("/");
}

void normalizeRGBtoMax(int max, int *red, int *green, int * blue) {
  double sum = *red + *green + *blue;
  if (sum < max) {
    return;
  }
  double ratio = max / sum;
  *red = *red * ratio;
  *green = *green * ratio;
  *blue = *blue * ratio;
}

// Parses file name from a string like http://10.0.1.47/index.html
String parseFileName(String url) {
  String ret_val = String("/index.html");
  int start = url.indexOf('/');
  if (start <= 0) return ret_val;
    
  int end = url.indexOf('?', start + 1);
  if (end <= 0) {
      end = url.indexOf(' ', start + 1);
      if (end <= 0) return ret_val;
  }
  if (end > start + 1) {
    return url.substring(start, end);
  }
  return ret_val;
}

// Parses numbers from a string like http://10.0.1.47/?r=76&g=0&b=255
void parseRGB(String url, char start_c, char term_c, int* red, int* green, int* blue) {
  int spot = url.indexOf(start_c);
  int start = url.indexOf('r', spot+1);
  int next = url.indexOf(term_c, start+1);
  *red = url.substring(start + 2, next).toInt();

  start = url.indexOf('g', next+1);
  next = url.indexOf(term_c, start+1);
  *green = url.substring(start + 2, next).toInt();

  start = url.indexOf('b', next+1);
  *blue = url.substring(start + 2).toInt();
}

RingBuffer::RingBuffer(int size) {
    values_ = new int16_t[size];
    size_ = size;
    next_ = 0;
}

void RingBuffer::Insert(int16_t value) {
    values_[next_] = value;
    next_ = NextIndex(next_);
}

int RingBuffer::PreviousIndex(int pos) {
    pos -= 1;
    if (pos < 0) {
        pos += size_;
    }
    return pos;
}

int RingBuffer::NextIndex(int pos) {
    pos += 1;
    if (pos > size_ - 1) {
        pos = 0;
    }
    return pos;
}

int RingBuffer::FindGT(int16_t value) {
    int steps = 1;
    for(int i = PreviousIndex(next_); i != next_; i = PreviousIndex(i)) {
        if (values_[i] > value) {
            return steps;
        }
        steps += 1;
    }
    return -1;
}

int RingBuffer::FindLT(int16_t value) {
    int steps = 1;
    for(int i = PreviousIndex(next_); i != next_; i = PreviousIndex(i)) {
        if (values_[i] < value) {
            return steps;
        }
        steps += 1;
    }
    return -1;
}