// wind_webserver.cpp

#include "Arduino.h"
#include "wind_webserver.h"

WindWebServer::WindWebServer(int port) {
  server_ = new WiFiServer(port);
  server_->begin();
  Serial.println("WindWebServer started");
}

WiFiClient* WindWebServer::CheckForClient() {
  if (!client_) {
    client_ = server_->available();
  }
  return &client_;
}

WiFiClient* WindWebServer::Client() {
  return &client_;
}

bool WindWebServer::ClientAvailable() {
  return client_ && client_.available();
}

bool WindWebServer::ClientConnected() {
  return client_ && client_.connected();
}

// Upon return, line_buffer_size_ == # chars written into line_buffer_.
void WindWebServer::BufferLine() {
  line_buffer_size_ = 0;
  bool found_cr = false;
  while (client_.connected()) {
    if (client_.available()) {
      if (line_buffer_size_ >= kMaxBufferSize) {
        line_buffer_[kMaxBufferSize - 1] = '\0';  // terminate c string
        return;
      }
      char c = client_.read();
      if (c == '\r') {
        found_cr = true;
        continue;
      }
      if (c == '\n' && found_cr) {
        line_buffer_[line_buffer_size_] = '\0';  // terminate c string
        return;
      }
      found_cr = false;
      line_buffer_[line_buffer_size_++] = c;
    }
  }
}

String WindWebServer::ReadLine() {
    BufferLine();
    return line_buffer_;
}

String WindWebServer::ReadPost() {
    int content_length = 0;
    while (client_.connected()) {
      if (client_.available()) {
        BufferLine();
        // Read lines until we find the empty line, marking the end of the header.
        if (line_buffer_size_ != 0) {
          Serial.print("Skipping header line=");
          Serial.println(line_buffer_);
          if (strncmp(line_buffer_, "Content-Length: ", 16) == 0) {
            content_length = atoi(line_buffer_ + 16);
          }
          continue; 
        }
        
        // Read actual content:    
        int i = 0;
        while ((i < content_length) && (i < kMaxBufferSize)) {
          delay(10);  // Helped w/ Pebble reliability.  Due to not checking client.available()?
          line_buffer_[i++] = client_.read();
        }
        line_buffer_[i] = '\0';
        line_buffer_size_ = i;
        return line_buffer_;
      }
   }
}

void WindWebServer::ReturnFile(const char* file_name) {        
  File f = SPIFFS.open(file_name, "r");
  int file_size = f.size();

  size_t num_sent = 0;
  size_t num_read = f.readBytes(line_buffer_, kMaxBufferSize-1);
  line_buffer_[num_read] = '\0';
  while (num_read > 0 ) {
    client_.print(line_buffer_);
    num_sent += num_read;
    num_read = f.readBytes(line_buffer_, kMaxBufferSize-1);
    line_buffer_[num_read] = '\0';
    yield();
  }
    
  client_.println("");
  client_.flush();
  client_.stop();
}
